@@ -965,7 +965,7 @@ static void output_acinclude(void)
 			printf("\tif test \"$CONFIG_ARM\" = \"y\"; then\n");
 		else if (strstr(tempdep->dir, "/parisc"))
 			printf("\tif test \"$CONFIG_PARISC\" = \"y\"; then\n");
-		else if (strstr(tempdep->name, "/pcmcia"))
+		else if (strstr(tempdep->dir, "/pcmcia"))
 			printf("\tif test \"$CONFIG_PCMCIA\" = \"y\"; then\n");
 		else
 			put_if = 0;
